unit UPascalCompiler;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, UPascalSource, UCodeProducer, UPascalParser, UAssemblerSource,
  UCSource;

type
  TCompiler = class
  private
    FOnErrorMessage: TOnErrorMessage;
    procedure ErrorMessage(Text: string);
  public
    ProgramCode: TProgram;
    SourceCode: TStringList;
    Parser: TPascalParser;
    Producer: TCodeProducer;
    constructor Create;
    procedure Compile;
    destructor Destroy; override;
    property OnErrorMessage: TOnErrorMessage read FOnErrorMessage write FOnErrorMessage;
  end;

implementation

{ TCompiler }

procedure TCompiler.Compile;
begin
  Parser.CodePosition := 1;
  TParserProgram(ProgramCode).Parse(Parser);
  Producer.Produce;
end;

constructor TCompiler.Create;
begin
  SourceCode := TStringList.Create;
  ProgramCode := TProgram.Create;
  Producer := TCProducer.Create;
  Producer.ProgramCode := ProgramCode;
  Parser := TPascalParser.Create;
  Parser.SourceCode := SourceCode;
  Parser.OnErrorMessage := ErrorMessage;
end;

destructor TCompiler.Destroy;
begin
  ProgramCode.Free;
  SourceCode.Free;
  Parser.Free;
  Producer.Free;
end;

procedure TCompiler.ErrorMessage(Text: string);
begin
  if Assigned(FOnErrorMessage) then FOnErrorMessage(Text);
end;

end.
